import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { is, fromJS } from 'immutable'
import { Spin, Empty, notification, message, Row, Col, Pagination, Modal } from 'antd'
import { DownOutlined } from '@ant-design/icons'

import Api from '@/api'
import Utils from '@/utils/utils.js'
import UtilsDM from '@/utils/utils-datamanage.js'
import preImg from '@/assets/img/prev.png'
import nextImg from '@/assets/img/next.png'
import MKEmitter from '@/utils/events.js'
import TimerTask from '@/utils/timer-task.js'
import asyncComponent from '@/utils/asyncComponent'
import './index.scss'

const CardItem = asyncComponent(() => import('../cardItem'))
const TableHeader = asyncComponent(() => import('../tableHeader'))
const MainAction = asyncComponent(() => import('@/tabviews/zshare/actionList'))
const NormalHeader = asyncComponent(() => import('@/tabviews/custom/components/share/normalheader'))

class DataCard extends Component {
  static propTpyes = {
    data: PropTypes.array,           // 统一查询数据
    config: PropTypes.object,        // 组件配置信息
    mainSearch: PropTypes.any,       // 外层搜索条件
  }

  state = {
    BID: '',                   // 上级ID
    BData: '',                 // 上级行数据
    config: null,              // 图表配置信息
    search: null,              // 搜索条件
    pageIndex: 1,              // 页码
    pageSize: 10,
    orderBy: '',
    pageOptions: [],
    activeKey: '',             // 选中卡
    selectKeys: [],            // 多选时选中卡片
    selectedData: [],          // 选中数据，用于工具栏按钮
    loading: false,            // 数据加载状态
    sync: false,               // 是否统一请求数据
    card: null,                // 卡片设置
    data: null,                // 数据
    total: null,
    precards: [],
    nextcards: [],
    selected: 'false',
    supNodes: [],
    supComs: null
  }

  loaded = false

  UNSAFE_componentWillMount () {
    const { data, initdata } = this.props
    let _config = fromJS(this.props.config).toJS()

    let BID = ''
    let BData = ''

    if (_config.setting.supModule) {
      BData = window.GLOB.CacheData.get(_config.setting.supModule)
    } else {
      BData = window.GLOB.CacheData.get(_config.$pageId)
    }
    if (BData) {
      BID = BData.$BID || ''
    }

    let _card = null
    let precards = []
    let nextcards = []

    if (_config.wrap.controlField) {
      if (_config.wrap.controlVal) {
        _config.wrap.controlVal = _config.wrap.controlVal.split(',')
      } else {
        _config.wrap.controlVal = ['']
      }
    }

    _config.$extend = false
    _config.$empty = true

    _config.subcards.forEach(item => {
      if (item.setting.click === 'button' && !item.setting.linkbtn) {
        item.elements.forEach(ele => {
          if (ele.eleType === 'button') {
            item.setting.linkbtn = ele.uuid
          }
        })
        if (!item.setting.linkbtn) {
          item.setting.click = ''
        }
      }

      if (item.$cardType !== 'extendCard') {
        _card = item
      } else if (!_card) {
        _config.$extend = true
        if (item.setting.width !== 24) {
          _config.$empty = false
        }
        precards.push(item)
      } else {
        _config.$extend = true
        _config.$empty = false
        nextcards.push(item)
      }
    })

    _config.subcards = null
    
    let _data = null
    let _sync = _config.setting.sync === 'true'

    if (_sync && data) {
      _data = data[_config.dataName] || []
      _sync = false
    } else if (_sync && initdata) {
      _data = initdata || []
      _sync = false
    }

    let selected = 'false'
    if (_config.wrap.selected === 'always' || _config.wrap.selected === 'init' || _config.wrap.selected === 'sign') {
      selected = _config.wrap.selected
    } else {
      _config.wrap.selected = 'false'
    }

    if (_data) {
      _data = _data.map((item, index) => {
        item.key = index
        item.$$uuid = item[_config.setting.primaryKey] || ''
        item.$$BID = BID || ''
        item.$$BData = BData || ''
        item.$Index = index + 1 + ''

        if (_config.wrap.controlField) {
          if (_config.wrap.controlVal.includes(item[_config.wrap.controlField] + '')) {
            item.$disabled = true
          }
        }
        return item
      })

      if (selected !== 'false') {
        setTimeout(() => {
          this.checkTopLine()
        }, 200)
        if (selected === 'init') {
          selected = 'false'
        }
      }
    }

    let supComs = null
    if (_config.wrap.supType === 'multi') {
      supComs = _config.supNodes.map(item => item.componentId)
    }

    _config.wrap.selStyle = _config.wrap.selStyle || 'active'
    _config.wrap.pagestyle = _config.wrap.pagestyle || 'page'
    _config.wrap.scale = _config.wrap.scale === 'true' ? 'scale' : ''
    _config.wrap.layout = (_config.wrap.layout || 'grid') + '-layout float-' + (_config.wrap.cardFloat || 'left')

    _config.wrap.wrapClass =  `${_config.wrap.selStyle} ${_config.wrap.cardType || ''} ${_config.wrap.scale}`

    this.loaded = _data !== null

    let pageOptions = ['10', '25', '50', '100', '500', '1000']

    if (!_config.setting.laypage) {
      _config.wrap.pagestyle = 'none'
    }

    if (_config.wrap.pagestyle === 'page') {
      let size = (_config.setting.pageSize || 10) + ''
      if (!pageOptions.includes(size)) {
        pageOptions.push(size)
        pageOptions = pageOptions.sort((a, b) => a - b)
      }
    }

    this.setState({
      pageSize: _config.setting.pageSize || 10,
      pageOptions,
      supComs,
      selected,
      precards,
      nextcards,
      sync: _sync,
      data: _data,
      BID: BID || '',
      BData: BData || '',
      config: _config,
      card: _card,
      search: Utils.initMainSearch(_config.search),
      arr_field: _config.columns.map(col => col.field).join(','),
    }, () => {
      if (_config.setting.sync !== 'true' && _config.setting.onload === 'true') {
        setTimeout(() => {
          this.loadData()
        }, _config.setting.delay || 0)
      }
    })
  }

  componentDidMount () {
    const { config } = this.state

    MKEmitter.addListener('reloadData', this.reloadData)
    MKEmitter.addListener('mkCheckAll', this.mkCheckAll)
    MKEmitter.addListener('resetSelectLine', this.resetParentParam)
    MKEmitter.addListener('queryModuleParam', this.queryModuleParam)
    MKEmitter.addListener('refreshLineData', this.refreshLineData)
    MKEmitter.addListener('refreshByButtonResult', this.refreshByButtonResult)
    
    if (config.timer) {
      this.timer = new TimerTask()
      this.timer.init(config.uuid, config.timer, config.timerRepeats, () => {
        this.setState({
          pageIndex: 1
        }, () => {
          this.loadData('', 'timer')
        })
      })
    }

    if (config.$cache && !this.loaded) {
      Api.getLCacheConfig(config.uuid).then(res => {
        if (!res || this.loaded) return
        let _data = res.map((item, index) => {
          item.key = index
          item.$$uuid = item[config.setting.primaryKey] || ''
          item.$Index = index + 1 + ''

          if (config.wrap.controlField) {
            if (config.wrap.controlVal.includes(item[config.wrap.controlField] + '')) {
              item.$disabled = true
            }
          }

          return item
        })

        this.setState({data: _data})
      })
    }
  }

  shouldComponentUpdate (nextProps, nextState) {
    return !is(fromJS(this.state), fromJS(nextState))
  }

  UNSAFE_componentWillReceiveProps (nextProps) {
    const { sync, config, BID, BData, selected } = this.state

    if (sync && !is(fromJS(this.props.data), fromJS(nextProps.data))) {
      let _data = []
      if (nextProps.data && nextProps.data[config.dataName]) {
        _data = nextProps.data[config.dataName] || []
        _data = _data.map((item, index) => {
          item.key = index
          item.$$uuid = item[config.setting.primaryKey] || ''
          item.$$BID = BID || ''
          item.$$BData = BData || ''
          item.$Index = index + 1 + ''

          if (config.wrap.controlField) {
            if (config.wrap.controlVal.includes(item[config.wrap.controlField] + '')) {
              item.$disabled = true
            }
          }

          return item
        })

        if (selected !== 'false') {
          setTimeout(() => {
            this.checkTopLine()
          }, 200)
          if (selected === 'init') {
            this.setState({selected: 'false'})
          }
        }
      }

      this.loaded = true

      this.setState({sync: false, data: _data})
    } else if (config.setting.useMSearch && nextProps.mainSearch && !is(fromJS(this.props.mainSearch), fromJS(nextProps.mainSearch))) {
      this.setState({pageIndex: 1}, () => {
        this.loadData()
      })
    }
  }

  componentWillUnmount () {
    this.setState = () => {
      return
    }
    MKEmitter.removeListener('reloadData', this.reloadData)
    MKEmitter.removeListener('mkCheckAll', this.mkCheckAll)
    MKEmitter.removeListener('resetSelectLine', this.resetParentParam)
    MKEmitter.removeListener('queryModuleParam', this.queryModuleParam)
    MKEmitter.removeListener('refreshLineData', this.refreshLineData)
    MKEmitter.removeListener('refreshByButtonResult', this.refreshByButtonResult)

    this.timer && this.timer.stop()
  }

  /**
   * @description 按钮执行完成后页面刷新
   * @param {*} menuId     // 菜单Id
   * @param {*} position   // 刷新位置
   * @param {*} btn        // 执行的按钮
   */
  refreshByButtonResult = (menuId, position, btn, id = '', lines) => {
    const { config, BID, supComs, supNodes } = this.state

    if (config.uuid !== menuId) return

    if (supComs) {
      if (position === 'mainline' || position === 'popclose') { // 主表刷新，去除同步刷新组件
        let supNode = supNodes[supNodes.length - 1]
        supComs.forEach((item, i) => {
          setTimeout(() => {
            if (supNode && supNode.key === item) {
              MKEmitter.emit('reloadData', item, supNode.value)
            } else {
              MKEmitter.emit('reloadData', item)
            }
          }, i * 10)
        })
      } else {
        if (position === 'line') {
          if (lines && lines.length === 1) {
            this.loadLinedata(lines[0].$$uuid)
          } else {
            this.loadData(id)
          }
        } else if (!btn || btn.resetPageIndex !== 'false') {
          this.setState({
            pageIndex: 1
          }, () => {
            this.loadData(id)
          })
        } else {
          this.loadData(id)
        }
      }
    } else {
      let supModule = config.setting.supModule

      if (position === 'line') {
        if (lines && lines.length === 1) {
          this.loadLinedata(lines[0].$$uuid)
        } else {
          this.loadData(id)
        }
      } else if ((position === 'mainline' || position === 'popclose') && supModule && BID) { // 刷新源组件时，附带刷新上级行与当前组件
        MKEmitter.emit('reloadData', supModule, BID)
      } else if (!btn || btn.resetPageIndex !== 'false') {
        this.setState({
          pageIndex: 1
        }, () => {
          this.loadData(id)
        })
      } else {
        this.loadData(id)
      }
    }
  }

  refreshLineData = (menuId, btn, uuid, count) => {
    const { config, data } = this.state

    if (config.uuid !== menuId) return
    
    let _data = fromJS(data).toJS().map(item => {
      if (item.$$uuid === uuid) {
        item[btn.field] = count
      }
      return item
    })

    this.setState({
      data: _data
    })
  }

  checkTopLine = (id) => {
    const { config, data, selected } = this.state

    if (!data || data.length === 0) {
      this.setState({
        activeKey: '',
        selectKeys: [],
        selectedData: []
      })
  
      MKEmitter.emit('resetSelectLine', config.uuid, '', '')
      if (config.setting.$hasSyncModule) {
        MKEmitter.emit('syncBalconyData', config.uuid, [], false)
      }
      return
    }

    if (selected === 'sign') {
      let index = ''
      let keys = []
      let items = []
      let last = ''
      data.forEach((item, i) => {
        if (!item.$disabled && item.selected === 'true') {
          items.push(item)
          keys.push(i)
          index = i
          last = item
        }
      })

      this.setState({
        activeKey: index,
        selectKeys: keys,
        selectedData: items
      })
  
      MKEmitter.emit('resetSelectLine', config.uuid, last ? last.$$uuid : '', last)
      if (config.setting.$hasSyncModule) {
        MKEmitter.emit('syncBalconyData', config.uuid, items, data.length === keys.length)
      }
      return
    }

    let index = 0
    if (id) {
      index = data.findIndex(item => item.$$uuid === id)
      if (index === -1) {
        index = 0
      }
    }

    if (data[index].$disabled) {
      this.setState({
        activeKey: '',
        selectKeys: [],
        selectedData: []
      })
  
      MKEmitter.emit('resetSelectLine', config.uuid, '', '')
      if (config.setting.$hasSyncModule) {
        MKEmitter.emit('syncBalconyData', config.uuid, [], false)
      }
      return
    }

    this.setState({
      activeKey: index,
      selectKeys: [index],
      selectedData: [data[index]]
    })

    MKEmitter.emit('resetSelectLine', config.uuid, data[index].$$uuid, data[index])
    if (config.setting.$hasSyncModule) {
      MKEmitter.emit('syncBalconyData', config.uuid, [data[index]], data.length === 1)
    }
  }

  mkCheckAll = (menuId, checked) => {
    const { config, data } = this.state

    if (config.uuid !== menuId) return

    if (checked) {
      let index = ''
      let keys = []
      let items = []
      let last = ''

      data.forEach((item, i) => {
        if (item.$disabled) return

        items.push(item)
        keys.push(i)
        index = i
        last = item
      })

      this.setState({
        activeKey: index,
        selectKeys: keys,
        selectedData: items
      })
  
      MKEmitter.emit('resetSelectLine', config.uuid, last ? last.$$uuid : '', last)
      MKEmitter.emit('syncBalconyData', config.uuid, items, data.length > 0 && data.length === keys.length)
      if (items.length === 0) {
        message.warning('未获取到数据！')
      }
    } else {
      this.setState({
        activeKey: '',
        selectKeys: [],
        selectedData: []
      })
  
      MKEmitter.emit('resetSelectLine', config.uuid, '', '')
      MKEmitter.emit('syncBalconyData', config.uuid, [], false)
    }
  }

  checkAll = () => {
    const { config, data, selectedData } = this.state

    if (config.wrap.cardType !== 'checkbox') return
    if (!data || data.length === 0) return
    
    if (selectedData.length === 0 || selectedData.length < data.length) {
      let index = ''
      let keys = []
      let items = []
      let last = ''

      data.forEach((item, i) => {
        if (item.$disabled) return

        items.push(item)
        keys.push(i)
        index = i
        last = item
      })

      this.setState({
        activeKey: index,
        selectKeys: keys,
        selectedData: items
      })
  
      MKEmitter.emit('resetSelectLine', config.uuid, last ? last.$$uuid : '', last)
      if (config.setting.$hasSyncModule) {
        MKEmitter.emit('syncBalconyData', config.uuid, items, data.length === keys.length)
      }
    } else {
      this.setState({
        activeKey: '',
        selectKeys: [],
        selectedData: []
      })
  
      MKEmitter.emit('resetSelectLine', config.uuid, '', '')
      if (config.setting.$hasSyncModule) {
        MKEmitter.emit('syncBalconyData', config.uuid, [], false)
      }
    }
  }

  reloadData = (menuId, id) => {
    const { config } = this.state

    if (config.uuid !== menuId) return

    if (!id) {
      this.loadData()
    } else {
      this.loadLinedata(id)
    }
  }

  resetParentParam = (MenuID, id, data) => {
    const { config, supComs } = this.state

    if (supComs) {
      if (!supComs.includes(MenuID)) return
      let supNodes = this.state.supNodes.filter(item => item.key !== MenuID)
      let bid = ''
      let _data = null

      if (id) {
        supNodes.push({key: MenuID, value: id, data})
      }
      if (supNodes.length > 0) {
        bid = supNodes[supNodes.length - 1].value
        _data = supNodes[supNodes.length - 1].data
      }

      if (bid !== this.state.BID || bid !== '') {
        this.setState({ BID: bid, BData: _data, pageIndex: 1, supNodes }, () => {
          this.loadData()
        })
      } else {
        this.setState({ supNodes })
      }
    } else {
      if (!config.setting.supModule || config.setting.supModule !== MenuID) return
      if (id !== this.state.BID || id !== '') {
        this.setState({ BID: id, BData: data, pageIndex: 1 }, () => {
          this.loadData()
        })
      }
    }
  }

  /**
   * @description 导出Excel时，获取页面搜索排序等参数
   */
  queryModuleParam = (menuId, callback) => {
    const { mainSearch } = this.props
    const { arr_field, config, search, orderBy } = this.state

    if (config.uuid !== menuId) return

    let searches = search ? fromJS(search).toJS() : []
    if (config.setting.useMSearch && mainSearch && mainSearch.length > 0) { // 主表搜索条件
      let keys = searches.map(item => item.key.toLowerCase())
      mainSearch.forEach(item => {
        if (!keys.includes(item.key.toLowerCase())) {
          searches.push(item)
        }
      })
    }

    callback({
      arr_field: arr_field,
      orderBy: orderBy ||config.setting.order || '',
      search: searches,
      menuName: config.name
    })
  }

  async loadData (id, type) {
    const { mainSearch } = this.props
    const { config, arr_field, pageIndex, pageSize, search, BID, BData, selected, orderBy } = this.state

    if (config.setting.supModule && !BID && config.wrap.supKey !== 'false') { // BID 不存在时，不做查询
      this.loaded = true

      this.setState({
        activeKey: '',
        selectKeys: [],
        selectedData: [],
        pageIndex: 1,
        data: [],
        total: 0,
        loading: false
      })
      
      if (selected !== 'false' || (id && config.wrap.selected !== 'false')) {
        setTimeout(() => {
          this.checkTopLine(id)
        }, 200)
        if (selected === 'init') {
          this.setState({selected: 'false'})
        }
      } else {
        MKEmitter.emit('resetSelectLine', config.uuid, '', '')
        if (config.setting.$hasSyncModule) {
          MKEmitter.emit('syncBalconyData', config.uuid, [], false)
        }
      }
      return
    }

    let searches = fromJS(search).toJS()
    if (config.setting.useMSearch && mainSearch && mainSearch.length > 0) { // 主表搜索条件
      let keys = searches.map(item => item.key.toLowerCase())
      mainSearch.forEach(item => {
        if (!keys.includes(item.key.toLowerCase())) {
          searches.push(item)
        }
      })
    }

    let requireFields = searches.filter(item => item.required && item.value === '')
    if (requireFields.length > 0) {
      return
    }

    if (type !== 'timer') {
      this.setState({
        loading: true
      })
    }

    let _orderBy = orderBy || config.setting.order || ''
    let param = UtilsDM.getQueryDataParams(config.setting, arr_field, searches, _orderBy, pageIndex, pageSize, BID)

    let result = await Api.genericInterface(param)
    if (result.status) {
      let start = 1
      if (config.setting.laypage) {
        start = pageSize * (pageIndex - 1) + 1
      }

      this.loaded = true
      if (config.$cache && pageIndex === 1 && config.setting.onload !== 'false') {
        Api.writeCacheConfig(config.uuid, result.data || '')
      }

      if (selected !== 'false' || (id && config.wrap.selected !== 'false')) {
        setTimeout(() => {
          this.checkTopLine(id)
        }, 200)
        if (selected === 'init') {
          this.setState({selected: 'false'})
        }
      } else {
        MKEmitter.emit('resetSelectLine', config.uuid, '', '')
        if (config.setting.$hasSyncModule) {
          MKEmitter.emit('syncBalconyData', config.uuid, [], false)
        }
      }

      let data = []

      if (type === 'plus') {
        let _data = (this.state.data || []).concat(result.data || [])
        data = _data.map((item, index) => {
          item.key = index
          item.$$uuid = item[config.setting.primaryKey] || ''
          item.$$BID = BID || ''
          item.$$BData = BData || ''
          item.$Index = index + 1 + ''

          if (config.wrap.controlField) {
            if (config.wrap.controlVal.includes(item[config.wrap.controlField] + '')) {
              item.$disabled = true
            }
          }
          return item
        })
      } else {
        data = result.data.map((item, index) => {
          item.key = index
          item.$$uuid = item[config.setting.primaryKey] || ''
          item.$$BID = BID || ''
          item.$$BData = BData || ''
          item.$Index = index + start + ''

          if (config.wrap.controlField) {
            if (config.wrap.controlVal.includes(item[config.wrap.controlField] + '')) {
              item.$disabled = true
            }
          }
          return item
        })
      }

      this.setState({
        activeKey: '',
        selectKeys: [],
        selectedData: [],
        data: data,
        total: result.total,
        loading: false
      })

      if (config.timer && config.clearField && result.data && result.data[0]) {
        let vals = (config.clearValue || '').split(',')
        if (vals.includes(result.data[0][config.clearField])) {
          this.timer && this.timer.stop()
        }
      }
      if (result.message) {
        if (result.ErrCode === 'Y') {
          Modal.success({
            title: result.message
          })
        } else if (result.ErrCode === 'S') {
          notification.success({
            top: 92,
            message: result.message,
            duration: 2
          })
        }
      }
    } else {
      this.setState({
        loading: false
      })
      this.timer && this.timer.stop()

      if (!result.message) return
      if (result.ErrCode === 'N') {
        Modal.error({
          title: result.message,
        })
      } else if (result.ErrCode !== '-2') {
        notification.error({
          top: 92,
          message: result.message,
          duration: 10
        })
      }
    }
  }

  /**
   * @description 获取单行数据
   */ 
  async loadLinedata (id) {
    const { mainSearch } = this.props
    const { config, arr_field, pageIndex, pageSize, search, BID, BData, orderBy } = this.state

    let searches = fromJS(search).toJS()
    if (config.setting.useMSearch && mainSearch && mainSearch.length > 0) { // 主表搜索条件
      let keys = searches.map(item => item.key.toLowerCase())
      mainSearch.forEach(item => {
        if (!keys.includes(item.key.toLowerCase())) {
          searches.push(item)
        }
      })
    }

    this.setState({
      loading: true
    })

    let _orderBy = orderBy || config.setting.order || ''
    let param = UtilsDM.getQueryDataParams(config.setting, arr_field, searches, _orderBy, pageIndex, pageSize, BID, id)

    let result = await Api.genericInterface(param)
    if (result.status) {
      let data = fromJS(this.state.data).toJS()
      if (result.data && result.data[0]) {
        let _data = result.data[0]

        try {
          data = data.map(item => {
            if (item[config.setting.primaryKey] === _data[config.setting.primaryKey]) {
              _data.key = item.key
              _data.$$uuid = _data[config.setting.primaryKey] || ''
              _data.$$BID = BID || ''
              _data.$$BData = BData || ''
              _data.$Index = item.$Index
              return _data
            } else {
              return item
            }
          })
        } catch (e) {
          console.warn('数据查询错误')
        }

        MKEmitter.emit('resetSelectLine', config.uuid, _data.$$uuid || '', _data)
      }

      this.setState({
        data: data,
        loading: false
      })
    } else {
      this.setState({
        loading: false
      })
      notification.error({
        top: 92,
        message: result.message,
        duration: 10
      })
    }
  }

  loadMore = () => {
    const { total, pageIndex, pageSize, loading } = this.state

    if (loading || pageSize * pageIndex >= total) {
      return
    }

    this.setState({
      pageIndex: pageIndex + 1
    }, () => {
      this.loadData('', 'plus')
    })
  }

  prevPage = () => {
    const { pageIndex } = this.state

    if (pageIndex === 1) return

    this.setState({
      pageIndex: pageIndex - 1
    }, () => {
      this.loadData()
    })
  }

  nextPage = () => {
    const { pageIndex, pageSize, total } = this.state
    let _total = pageSize * pageIndex

    if (_total >= total) return

    this.setState({
      pageIndex: pageIndex + 1
    }, () => {
      this.loadData()
    })
  }

  changePageIndex = (page) => {
    this.setState({
      pageIndex: page
    }, () => {
      this.loadData()
    })
  }

  refreshSearch = (list) => {
    this.setState({
      search: list,
      pageIndex: 1
    }, () => {
      this.loadData()
    })
  }

  pageSizeChange = (current, size) => {
    this.setState({
      pageIndex: current,
      pageSize: size
    }, () => {
      this.loadData()
    })
  }
  
  changeCard = (index, item) => {
    const { config, selectKeys, selectedData, activeKey, data } = this.state

    if (!config.wrap.cardType) return
    if (item.$disabled) return
    
    let _selectKeys = []
    let _selectedData = []
    let _activeKey = ''
    let _item = item
    if (config.wrap.cardType === 'checkbox') {
      if (activeKey === index) {
        _selectKeys = selectKeys.filter(key => key !== index)
        _selectedData = selectedData.filter(cell => cell.key !== index)
        _activeKey = _selectKeys.slice(-1)[0]
        _item = _selectedData.slice(-1)[0] || ''
      } else if (selectKeys.indexOf(index) > -1) {
        _selectKeys = selectKeys.filter(key => key !== index)
        _selectedData = selectedData.filter(cell => cell.key !== index)
        _activeKey = activeKey
        _item = _selectedData.filter(cell => cell.key === activeKey)[0] || ''
      } else {
        _selectKeys = [...selectKeys, index]
        _selectedData = [...selectedData, item]
        _activeKey = index
      }
    } else {
      if (activeKey === index) return

      _selectedData = [item]
      _activeKey = index
    }

    this.setState({
      activeKey: _activeKey,
      selectKeys: _selectKeys,
      selectedData: _selectedData
    })

    MKEmitter.emit('resetSelectLine', config.uuid, (_item ? _item.$$uuid : ''), _item)
    if (config.setting.$hasSyncModule) {
      MKEmitter.emit('syncBalconyData', config.uuid, _selectedData, data.length === _selectedData.length)
    }
  }

  refreshByHeader = (sorter) => {
    this.setState({
      orderBy: sorter || ''
    }, () => {
      this.loadData()
    })
  }

  render() {
    const { config, precards, nextcards, loading, data, pageIndex, pageSize, total, card, activeKey, BID, BData, selectedData, selectKeys } = this.state

    if (config.wrap.empty === 'hidden' && (!data || data.length === 0)) return null

    let _total = 0
    let switchable = false
    if (config.wrap.pagestyle === 'switch' && total > pageSize && data) {
      _total = pageSize * pageIndex
      switchable = true
    }

    let extendData = null

    if (config.$extend) {
      extendData = {$$BID: BID, $$BData: BData, $$selectedData: selectedData, $$type: 'extendCard'}
      
      if (data && data[0]) {
        if (selectedData[0]) {
          extendData = {...extendData, ...selectedData[0]}
        } else {
          extendData = {...extendData, ...data[0]}
        }
      }
    }

    let checkAll = ''
    if (config.wrap.selStyle && config.wrap.selStyle.indexOf('check') > -1) {
      if (selectedData.length > 0) {
        checkAll = selectedData.length < data.length ? ' half' : ' whole'
      }
    }

    return (
      <div className="custom-data-card-box" id={'anchor' + config.uuid} style={config.style}>
        {loading ?
          <div className="loading-mask">
            {data ? <div className="ant-spin-blur"></div> : null}
            <Spin />
          </div> : null
        }
        <NormalHeader config={config} BID={BID} refresh={this.refreshSearch} />
        {config.action && config.action.length > 0 ?
          <MainAction
            BID={BID}
            BData={BData}
            setting={config.setting}
            actions={config.action}
            columns={config.columns}
            selectedData={selectedData}
          /> : null
        }
        <div className={`data-zoom ${config.wrap.wrapClass}`}>
          {switchable ? <div className={'prev-page ' + (pageIndex === 1 ? 'disabled' : '')} onClick={this.prevPage}><div><div><img src={preImg} alt=""/></div></div></div> : null}
          <Row className={'card-row-list ' + config.wrap.layout}>
            {precards.map((item, index) => (
              <Col key={'pre' + index} className="extend-card" span={item.setting.width || 6}>
                {item.setting.cardRole === 'header' ? <TableHeader card={item} data={extendData} refresh={this.refreshByHeader}>
                  {item.setting.checkAll === 'show' ? <span onClick={this.checkAll} className={'circle-select' + checkAll}></span> : null}
                </TableHeader> : <CardItem card={item} cards={config} data={extendData}>
                  {item.setting.checkAll === 'show' ? <span onClick={this.checkAll} className={'circle-select' + checkAll}></span> : null}
                </CardItem>}
              </Col>
            ))}
            {data && data.map((item, index) => {
              let className = 'mk-card '
              if (config.wrap.parity === 'true') {
                if (index % 2 === 1) {
                  className += 'mk-parity-bg '
                }
              }
              if (item.$disabled) {
                className = 'mk-disabled'
              } else if (activeKey === index) {
                className += 'active'
              } else if (selectKeys.indexOf(index) > -1) {
                className += 'selected'
              }

              return (
                <Col className={className} key={index} span={card.setting.width}>
                  <CardItem card={card} cards={config} data={item} onClick={() => {this.changeCard(index, item)}}>
                    <span className="circle-select"></span>
                  </CardItem>
                </Col>
              )
            })}
            {nextcards.map((item, index) => (
              <Col key={'next' + index} className="extend-card" span={item.setting.width || 6}>
                {item.setting.cardRole === 'header' ? <TableHeader card={item} data={extendData} refresh={this.refreshByHeader}>
                  {item.setting.checkAll === 'show' ? <span onClick={this.checkAll} className={'circle-select' + checkAll}></span> : null}
                </TableHeader> : <CardItem card={item} cards={config} data={extendData}>
                  {item.setting.checkAll === 'show' ? <span onClick={this.checkAll} className={'circle-select' + checkAll}></span> : null}
                </CardItem>}
              </Col>
            ))}
          </Row>
          {switchable ? <div className={'prev-page ' + (total <= _total ? 'disabled' : '')} onClick={this.nextPage}><div><div><img src={nextImg} alt=""/></div></div></div> : null}
        </div>
        {config.$empty && (!data || data.length === 0) ? <Empty description={false}/> : null}
        {config.wrap.pagestyle === 'page' && data ? <Pagination size="small" total={total} showTotal={(t, range) => total > 0 ? `${range[0]}-${range[1]} 共 ${total} 条` : `共 ${total} 条`} pageSize={pageSize} showSizeChanger={true} pageSizeOptions={this.state.pageOptions} onChange={this.changePageIndex} onShowSizeChange={this.pageSizeChange} current={pageIndex}/> : null}
        {config.wrap.pagestyle === 'more' && data && data.length > 0 ? <div className={'mk-more' + (pageSize * pageIndex >= total ? ' disabled' : '')} onClick={this.loadMore}>查看更多<DownOutlined/></div> : null}
      </div>
    )
  }
}

export default DataCard